%%************************
% Matlab Toolbox for L0.5 norm-based sparse sampling and reconstruction of self-sparse NMR Spectra.
% Code for the following papers:
% [1] Xiaobo Qu, Di Guo, Xue Cao, Shuhui Cai, Zhong Chen. Reconstruction of self-sparse 2D NMR spectra from undersampled data in the indirect dimension, Sensors, 11(9):8888-8909,2011.
% [2] Xiaobo Qu, Weiru Zhang, Di Guo, Congbo Cai, Shuhui Cai, Zhong Chen. Iterative thresholding compressed sensing MRI based on contourlet transform, Inverse Problems in Science and Engineering, 18(6):737-758, 2010.
% See "readme" for details

% Author: Xiaobo Qu, PhD
% Homepage: http://www.quxiaobo.org
% Email: quxiaobo <|at|> xmu.edu.cn
% Affiliations:
% (a) Department of Electronic Science, Fujian Key Laboratory of Plasma and Magnetic Resonance,Xiamen University, Xiamen 361005, China
% (b) Department of Communication Engineering, Xiamen University, Xiamen 361005, China

%%************************
%%
clc;
clear all;
close all;
clear classes;
%%
Path_SparseMRS
%% parameter settings
% ExpIST
ExpIST.Relative_Tol=1e-8;% relative residue, parameter eta in Ref.[2].  
ExpIST.DecayThr=0.95;% decacy factor of the threshold in Ref. [2].
% PSOCA
PSOCA.lamda=10^8;% regularization parameter, see Ref.[1]
PSOCA.p_norm=1/2;% value of p in Lp norm, see Ref.[1]
%% load the NMR spectra
load data_CH_COSY
%% convert the spectra to FID data
FID=ifft2c(MRS);
%%
[row,column]=size(MRS);
%% show the sampling pattern,the white lines corresponding to the sampled FID data
% figure,imshow(mask)
%% define the undersampling operator, and obtain the undersampled FID data
Fu=Fu_downsample(mask,row,column);
y=FID(logical(mask));
%%
tic
MRS_zerofilling=Fu'*y;
time_zerofilling=toc
MRS_zerofilling_max=max(abs(MRS_zerofilling(:)));
MRS_zerofilling=MRS_zerofilling./MRS_zerofilling_max;
%%
UI=UnitI;% Use identity matrix to sparsify the NMR spectra
WT=Wavelet_mdwt('Symmlet', 8, 4); % Use the wavelet to sparsify the NMR spectra
%
tic
MRS_rec_UintI_ExpIST=solver_ExpIST_MRS(y,Fu,UI,ExpIST);% Soft thresholding with exponential decay of threshod
time_UintI_ExpIST=toc
MRS_rec_UintI_ExpIST_max=max(abs(MRS_rec_UintI_ExpIST(:)));
MRS_rec_UintI_ExpIST=MRS_rec_UintI_ExpIST/MRS_rec_UintI_ExpIST_max;
%%
tic
MRS_rec_WT_ExpIST=solver_ExpIST_MRS(y,Fu,WT,ExpIST);
time_WT_ExpIST=toc
MRS_rec_WT_ExpIST_max=max(abs(MRS_rec_WT_ExpIST(:)));
MRS_rec_WT_ExpIST=MRS_rec_WT_ExpIST/MRS_rec_WT_ExpIST_max;
%%
paramsUILp=PSOCA;
paramsUILp.psi=UI;
paramsUILp.p_norm=1;
tic
MRS_rec_UintI_L1=solver_PSOCA_MRS(y,mask,paramsUILp);
time_UI_L1=toc
MRS_rec_UintI_L1_max=max(abs(MRS_rec_UintI_L1(:)));
MRS_rec_UintI_L1=MRS_rec_UintI_L1/MRS_rec_UintI_L1_max;
%%
paramsUILp=PSOCA;
paramsUILp.psi=UI;
paramsUILp.p_norm=1/2;
tic
MRS_rec_UintI_L05=solver_PSOCA_MRS(y,mask,paramsUILp);
time_UI_L05=toc
MRS_rec_UintI_L05_max=max(abs(MRS_rec_UintI_L05(:)));
MRS_rec_UintI_L05=MRS_rec_UintI_L05/MRS_rec_UintI_L05_max;
%%
paramsWTLp=PSOCA;
paramsWTLp.psi=WT;
paramsWTLp.p_norm=1;
tic
MRS_rec_WT_L1=solver_PSOCA_MRS(y,mask,paramsWTLp);
time_WT_L1=toc
MRS_rec_WT_L1_max=max(abs(MRS_rec_WT_L1(:)));
MRS_rec_WT_L1=MRS_rec_WT_L1/MRS_rec_WT_L1_max;
%%
paramsWTLp=PSOCA;
paramsWTLp.psi=WT;
paramsWTLp.p_norm=1/2;
tic
MRS_rec_WT_L05=solver_PSOCA_MRS(y,mask,paramsWTLp);
time_WT_L05=toc
MRS_rec_WT_L05_max=max(abs(MRS_rec_WT_L05(:)));
MRS_rec_WT_L05=MRS_rec_WT_L05/MRS_rec_WT_L05_max;
%%
supress_value=0; % Do not supress small spectra peaks
RLNE_zerofilling=RLNE(MRS,MRS_zerofilling,supress_value);
RLNE_ui_ExpIST=RLNE(MRS,MRS_rec_UintI_ExpIST,supress_value);
RLNE_WT_ExpIST=RLNE(MRS,MRS_rec_WT_ExpIST,supress_value);
RLNE_ui_L1=RLNE(MRS,MRS_rec_UintI_L1,supress_value);
RLNE_ui_L05=RLNE(MRS,MRS_rec_UintI_L05,supress_value);
RLNE_WT_L1=RLNE(MRS,MRS_rec_WT_L1,supress_value);
RLNE_WT_L05=RLNE(MRS,MRS_rec_WT_L05,supress_value);

disp('********Relative L2 norm error without supressing********')
disp('Zerofilling')
disp(RLNE_zerofilling)
disp('Wavelet: IST-L1 PSOCA-L1 PSOCA L0.5')
disp([             RLNE_WT_ExpIST RLNE_WT_L1 RLNE_WT_L05])
disp('Identity matrix: IST-L1 PSOCA-L1 PSOCA L0.5')
disp([                     RLNE_ui_ExpIST RLNE_ui_L1 RLNE_ui_L05])
%%
supress_value=0.1; % Supress small spectra peaks
RLNE_zerofilling_sp=RLNE(MRS,MRS_zerofilling,supress_value);
RLNE_ui_ExpIST_sp=RLNE(MRS,MRS_rec_UintI_ExpIST,supress_value);
RLNE_WT_ExpIST_sp=RLNE(MRS,MRS_rec_WT_ExpIST,supress_value);
RLNE_ui_L1_sp=RLNE(MRS,MRS_rec_UintI_L1,supress_value);
RLNE_ui_L05_sp=RLNE(MRS,MRS_rec_UintI_L05,supress_value);
RLNE_WT_L1_sp=RLNE(MRS,MRS_rec_WT_L1,supress_value);
RLNE_WT_L05_sp=RLNE(MRS,MRS_rec_WT_L05,supress_value);
disp('********Relative L2 norm error with supressing********')
disp('Zerofilling')
disp(RLNE_zerofilling_sp)
disp('Wavelet: IST-L1 PSOCA-L1 PSOCA L0.5')
disp([             RLNE_WT_ExpIST_sp RLNE_WT_L1_sp RLNE_WT_L05_sp])
disp('Identity matrix: IST-L1 PSOCA-L1 PSOCA L0.5')
disp([                     RLNE_ui_ExpIST_sp RLNE_ui_L1_sp RLNE_ui_L05_sp])
%%
close all
contourLevel=10;
figure,contour(abs(MRS),contourLevel),title('Fully sampled MRS'),colorbar
figure,contour(abs(MRS_zerofilling),contourLevel),title('Zerofilling MRS'),colorbar
figure,contour(abs(MRS_rec_UintI_ExpIST),contourLevel),title('MRS reconstructed using wavelet-based IST L1'),colorbar
figure,contour(abs(MRS_rec_WT_ExpIST),contourLevel),title('MRS reconstructed using identity matrix-based IST L1'),colorbar
figure,contour(abs(MRS_rec_WT_L1),contourLevel),title('MRS reconstructed using wavelet-based PSOCA L1'),colorbar
figure,contour(abs(MRS_rec_WT_L05),contourLevel),title('MRS reconstructed using wavelet-based PSOCA L0.5'),colorbar
figure,contour(abs(MRS_rec_UintI_L1),contourLevel),title('MRS reconstructed using identity matrix-based PSOCA L1'),colorbar
figure,contour(abs(MRS_rec_UintI_L05),contourLevel),title('MRS reconstructed using identity matrix-based PSOCA L0.5'),colorbar